@@ -1,40 +1,31 @@
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 import logging
 import os
 import re
 import tarfile
-import time
 
 from collections import defaultdict
 from contextlib import contextmanager
 from copy import copy
 from gzip import GzipFile
 from io import BytesIO
+from pathlib import Path
 from posixpath import join as pjoin
 from pprint import pformat
-from tarfile import TarInfo
 from typing import TYPE_CHECKING
-from typing import Dict
 from typing import Iterator
-from typing import List
-from typing import Optional
-from typing import Set
-from typing import Tuple
 
-from poetry.core.utils._compat import Path
-from poetry.core.utils._compat import decode
-from poetry.core.utils._compat import encode
-from poetry.core.utils._compat import to_str
-
-from ..utils.helpers import normalize_file_permissions
-from ..utils.package_include import PackageInclude
-from .builder import Builder
-from .builder import BuildIncludeFile
+from poetry.core.masonry.builders.builder import Builder
+from poetry.core.masonry.builders.builder import BuildIncludeFile
 
 
 if TYPE_CHECKING:
-    from poetry.core.packages import Dependency  # noqa
-    from poetry.core.packages import ProjectPackage  # noqa
+    from tarfile import TarInfo
+
+    from poetry.core.masonry.utils.package_include import PackageInclude
+    from poetry.core.packages.dependency import Dependency
+    from poetry.core.packages.project_package import ProjectPackage
 
 SETUP = """\
 # -*- coding: utf-8 -*-
@@ -62,27 +53,26 @@ logger = logging.getLogger(__name__)
 
 
 class SdistBuilder(Builder):
-
     format = "sdist"
 
-    def build(self, target_dir=None):  # type: (Optional[Path]) -> Path
+    def build(
+        self,
+        target_dir: Path | None = None,
+    ) -> Path:
         logger.info("Building <info>sdist</info>")
-        if target_dir is None:
-            target_dir = self._path / "dist"
+        target_dir = target_dir or self.default_target_dir
 
         if not target_dir.exists():
             target_dir.mkdir(parents=True)
 
-        target = target_dir / "{}-{}.tar.gz".format(
-            self._package.pretty_name, self._meta.version
-        )
+        target = target_dir / f"{self._package.pretty_name}-{self._meta.version}.tar.gz"
         gz = GzipFile(target.as_posix(), mode="wb", mtime=0)
         tar = tarfile.TarFile(
             target.as_posix(), mode="w", fileobj=gz, format=tarfile.PAX_FORMAT
         )
 
         try:
-            tar_dir = "{}-{}".format(self._package.pretty_name, self._meta.version)
+            tar_dir = f"{self._package.pretty_name}-{self._meta.version}"
 
             files_to_add = self.find_files_to_add(exclude_build=False)
 
@@ -103,32 +93,36 @@ class SdistBuilder(Builder):
                 setup = self.build_setup()
                 tar_info = tarfile.TarInfo(pjoin(tar_dir, "setup.py"))
                 tar_info.size = len(setup)
-                tar_info.mtime = time.time()
+                tar_info.mtime = 0
+                tar_info = self.clean_tarinfo(tar_info)
                 tar.addfile(tar_info, BytesIO(setup))
 
             pkg_info = self.build_pkg_info()
 
             tar_info = tarfile.TarInfo(pjoin(tar_dir, "PKG-INFO"))
             tar_info.size = len(pkg_info)
-            tar_info.mtime = time.time()
+            tar_info.mtime = 0
+            tar_info = self.clean_tarinfo(tar_info)
             tar.addfile(tar_info, BytesIO(pkg_info))
         finally:
             tar.close()
             gz.close()
 
-        logger.info("Built <comment>{}</comment>".format(target.name))
+        logger.info(f"Built <comment>{target.name}</comment>")
         return target
 
-    def build_setup(self):  # type: () -> bytes
+    def build_setup(self) -> bytes:
+        from poetry.core.masonry.utils.package_include import PackageInclude
+
         before, extra, after = [], [], []
-        package_dir = {}
+        package_dir: dict[str, str] = {}
 
         # If we have a build script, use it
         if self._package.build_script:
-            after += [
-                "from {} import *".format(self._package.build_script.split(".")[0]),
-                "build(setup_kwargs)",
-            ]
+            import_name = ".".join(
+                Path(self._package.build_script).with_suffix("").parts
+            )
+            after += [f"from {import_name} import *", "build(setup_kwargs)"]
 
         modules = []
         packages = []
@@ -142,7 +136,14 @@ class SdistBuilder(Builder):
                     pkg_dir, _packages, _package_data = self.find_packages(include)
 
                     if pkg_dir is not None:
-                        package_dir[""] = os.path.relpath(pkg_dir, str(self._path))
+                        pkg_root = os.path.relpath(pkg_dir, str(self._path))
+                        if "" in package_dir:
+                            package_dir.update(
+                                (p, os.path.join(pkg_root, p.replace(".", "/")))
+                                for p in _packages
+                            )
+                        else:
+                            package_dir[""] = pkg_root
 
                     packages += [p for p in _packages if p not in packages]
                     package_data.update(_package_data)
@@ -158,63 +159,65 @@ class SdistBuilder(Builder):
                 pass
 
         if package_dir:
-            before.append("package_dir = \\\n{}\n".format(pformat(package_dir)))
+            before.append(f"package_dir = \\\n{pformat(package_dir)}\n")
             extra.append("'package_dir': package_dir,")
 
         if packages:
-            before.append("packages = \\\n{}\n".format(pformat(sorted(packages))))
+            before.append(f"packages = \\\n{pformat(sorted(packages))}\n")
             extra.append("'packages': packages,")
 
         if package_data:
-            before.append("package_data = \\\n{}\n".format(pformat(package_data)))
+            before.append(f"package_data = \\\n{pformat(package_data)}\n")
             extra.append("'package_data': package_data,")
 
         if modules:
-            before.append("modules = \\\n{}".format(pformat(modules)))
-            extra.append("'py_modules': modules,".format())
+            before.append(f"modules = \\\n{pformat(modules)}")
+            extra.append("'py_modules': modules,")
 
         dependencies, extras = self.convert_dependencies(
             self._package, self._package.requires
         )
         if dependencies:
-            before.append(
-                "install_requires = \\\n{}\n".format(pformat(sorted(dependencies)))
-            )
+            before.append(f"install_requires = \\\n{pformat(sorted(dependencies))}\n")
             extra.append("'install_requires': install_requires,")
 
         if extras:
-            before.append("extras_require = \\\n{}\n".format(pformat(extras)))
+            before.append(f"extras_require = \\\n{pformat(extras)}\n")
             extra.append("'extras_require': extras_require,")
 
         entry_points = self.convert_entry_points()
         if entry_points:
-            before.append("entry_points = \\\n{}\n".format(pformat(entry_points)))
+            before.append(f"entry_points = \\\n{pformat(entry_points)}\n")
             extra.append("'entry_points': entry_points,")
 
+        script_files = self.convert_script_files()
+        if script_files:
+            rel_paths = [str(p.relative_to(self._path)) for p in script_files]
+            before.append(f"scripts = \\\n{pformat(rel_paths)}\n")
+            extra.append("'scripts': scripts,")
+
         if self._package.python_versions != "*":
             python_requires = self._meta.requires_python
 
-            extra.append("'python_requires': {!r},".format(python_requires))
+            extra.append(f"'python_requires': {python_requires!r},")
 
-        return encode(
-            SETUP.format(
-                before="\n".join(before),
-                name=to_str(self._meta.name),
-                version=to_str(self._meta.version),
-                description=to_str(self._meta.summary),
-                long_description=to_str(self._meta.description),
-                author=to_str(self._meta.author),
-                author_email=to_str(self._meta.author_email),
-                maintainer=to_str(self._meta.maintainer),
-                maintainer_email=to_str(self._meta.maintainer_email),
-                url=to_str(self._meta.home_page),
-                extra="\n    ".join(extra),
-                after="\n".join(after),
-            )
-        )
+        return SETUP.format(
+            before="\n".join(before),
+            name=str(self._meta.name),
+            version=str(self._meta.version),
+            description=str(self._meta.summary),
+            long_description=str(self._meta.description),
+            author=str(self._meta.author),
+            author_email=str(self._meta.author_email),
+            maintainer=str(self._meta.maintainer),
+            maintainer_email=str(self._meta.maintainer_email),
+            url=str(self._meta.home_page),
+            extra="\n    ".join(extra),
+            after="\n".join(after),
+        ).encode()
 
     @contextmanager
-    def setup_py(self):  # type: () -> Iterator[Path]
+    def setup_py(self) -> Iterator[Path]:
         setup = self._path / "setup.py"
         has_setup = setup.exists()
 
@@ -222,19 +225,19 @@ class SdistBuilder(Builder):
             logger.warning("A setup.py file already exists. Using it.")
         else:
             with setup.open("w", encoding="utf-8") as f:
-                f.write(decode(self.build_setup()))
+                f.write(self.build_setup().decode())
 
         yield setup
 
         if not has_setup:
             setup.unlink()
 
-    def build_pkg_info(self):  # type: () -> bytes
-        return encode(self.get_metadata_content())
+    def build_pkg_info(self) -> bytes:
+        return self.get_metadata_content().encode()
 
     def find_packages(
-        self, include
-    ):  # type: (PackageInclude) -> Tuple[str, List[str], dict]
+        self, include: PackageInclude
+    ) -> tuple[str | None, list[str], dict[str, list[str]]]:
         """
         Discover subpackages and data.
 
@@ -247,14 +250,14 @@ class SdistBuilder(Builder):
         base = str(include.elements[0].parent)
 
         pkg_name = include.package
-        pkg_data = defaultdict(list)
+        pkg_data: dict[str, list[str]] = defaultdict(list)
         # Undocumented distutils feature:
         # the empty string matches all package names
         pkg_data[""].append("*")
         packages = [pkg_name]
         subpkg_paths = set()
 
-        def find_nearest_pkg(rel_path):  # type: (str) -> Tuple[str, str]
+        def find_nearest_pkg(rel_path: str) -> tuple[str, str]:
             parts = rel_path.split(os.sep)
             for i in reversed(range(1, len(parts))):
                 ancestor = "/".join(parts[:i])
@@ -265,7 +268,7 @@ class SdistBuilder(Builder):
             # Relative to the top-level package
             return pkg_name, Path(rel_path).as_posix()
 
-        for path, dirnames, filenames in os.walk(str(base), topdown=True):
+        for path, _dirnames, filenames in os.walk(str(base), topdown=True):
             if os.path.basename(path) == "__pycache__":
                 continue
 
@@ -313,37 +316,36 @@ class SdistBuilder(Builder):
 
         return pkgdir, sorted(packages), pkg_data
 
-    def find_files_to_add(
-        self, exclude_build=False
-    ):  # type: (bool) -> Set[BuildIncludeFile]
-        to_add = super(SdistBuilder, self).find_files_to_add(exclude_build)
+    def find_files_to_add(self, exclude_build: bool = False) -> set[BuildIncludeFile]:
+        to_add = super().find_files_to_add(exclude_build)
 
         # add any additional files, starting with all LICENSE files
-        additional_files = {
-            license_file for license_file in self._path.glob("LICENSE*")
-        }
+        additional_files = set(self._path.glob("LICENSE*"))
+
+        # add script files
+        additional_files.update(self.convert_script_files())
 
         # Include project files
-        additional_files.add("pyproject.toml")
+        additional_files.add(Path("pyproject.toml"))
 
         # add readme if it is specified
         if "readme" in self._poetry.local_config:
             additional_files.add(self._poetry.local_config["readme"])
 
-        for file in additional_files:
+        for additional_file in additional_files:
             file = BuildIncludeFile(
-                path=file, project_root=self._path, source_root=self._path
+                path=additional_file, project_root=self._path, source_root=self._path
             )
             if file.path.exists():
-                logger.debug("Adding: {}".format(file.relative_to_source_root()))
+                logger.debug(f"Adding: {file.relative_to_source_root()}")
                 to_add.add(file)
 
         return to_add
 
     @classmethod
     def convert_dependencies(
-        cls, package, dependencies
-    ):  # type: ("ProjectPackage", List["Dependency"]) -> Tuple[List[str], Dict[str, List[str]]]
+        cls, package: ProjectPackage, dependencies: list[Dependency]
+    ) -> tuple[list[str], dict[str, list[str]]]:
         main = []
         extras = defaultdict(list)
         req_regex = re.compile(r"^(.+) \((.+)\)$")
@@ -353,9 +355,7 @@ class SdistBuilder(Builder):
                 for extra_name, reqs in package.extras.items():
                     for req in reqs:
                         if req.name == dependency.name:
-                            requirement = to_str(
-                                dependency.to_pep_508(with_extras=False)
-                            )
+                            requirement = dependency.to_pep_508(with_extras=False)
                             if ";" in requirement:
                                 requirement, conditions = requirement.split(";")
 
@@ -379,7 +379,7 @@ class SdistBuilder(Builder):
                             extras[extra_name].append(requirement)
                 continue
 
-            requirement = to_str(dependency.to_pep_508())
+            requirement = dependency.to_pep_508()
             if ";" in requirement:
                 requirement, conditions = requirement.split(";")
 
@@ -400,7 +400,7 @@ class SdistBuilder(Builder):
         return main, dict(extras)
 
     @classmethod
-    def clean_tarinfo(cls, tar_info):  # type: (TarInfo) -> TarInfo
+    def clean_tarinfo(cls, tar_info: TarInfo) -> TarInfo:
         """
         Clean metadata from a TarInfo object to make it more reproducible.
 
@@ -409,6 +409,8 @@ class SdistBuilder(Builder):
             - Normalise permissions to 644 or 755
             - Set mtime if not None
         """
+        from poetry.core.masonry.utils.helpers import normalize_file_permissions
+
         ti = copy(tar_info)
         ti.uid = 0
         ti.gid = 0
