@@ -1,4 +1,4 @@
-from __future__ import unicode_literals
+from __future__ import annotations
 
 import contextlib
 import csv
@@ -12,35 +12,30 @@ import tempfile
 import zipfile
 
 from base64 import urlsafe_b64encode
-from io import BytesIO
 from io import StringIO
+from pathlib import Path
 from typing import TYPE_CHECKING
 from typing import Iterator
-from typing import Optional
 from typing import TextIO
-from typing import Union
 
 from packaging.tags import sys_tags
 
 from poetry.core import __version__
-from poetry.core.semver import parse_constraint
-from poetry.core.utils._compat import PY2
-from poetry.core.utils._compat import Path
-from poetry.core.utils._compat import decode
-
-from ..utils.helpers import escape_name
-from ..utils.helpers import escape_version
-from ..utils.helpers import normalize_file_permissions
-from .builder import Builder
-from .sdist import SdistBuilder
+from poetry.core.masonry.builders.builder import Builder
+from poetry.core.masonry.builders.sdist import SdistBuilder
+from poetry.core.masonry.utils.helpers import escape_name
+from poetry.core.masonry.utils.helpers import escape_version
+from poetry.core.masonry.utils.helpers import normalize_file_permissions
+from poetry.core.masonry.utils.package_include import PackageInclude
+from poetry.core.semver.helpers import parse_constraint
 
 
 if TYPE_CHECKING:
-    from poetry.core.poetry import Poetry  # noqa
+    from poetry.core.poetry import Poetry
 
 wheel_file_template = """\
 Wheel-Version: 1.0
-Generator: poetry {version}
+Generator: poetry-core {version}
 Root-Is-Purelib: {pure_lib}
 Tag: {tag}
 """
@@ -52,38 +47,53 @@ class WheelBuilder(Builder):
     format = "wheel"
 
     def __init__(
-        self, poetry, target_dir=None, original=None, executable=None
-    ):  # type: ("Poetry", Optional[Path], Optional[Path], Optional[str]) -> None
-        super(WheelBuilder, self).__init__(poetry, executable=executable)
+        self,
+        poetry: Poetry,
+        original: Path | None = None,
+        executable: Path | None = None,
+        editable: bool = False,
+    ) -> None:
+        super().__init__(poetry, executable=executable)
 
-        self._records = []
+        self._records: list[tuple[str, str, int]] = []
         self._original_path = self._path
-        self._target_dir = target_dir or (self._poetry.file.parent / "dist")
         if original:
-            self._original_path = original.file.parent
+            self._original_path = original.parent
+        self._editable = editable
 
     @classmethod
     def make_in(
-        cls, poetry, directory=None, original=None, executable=None
-    ):  # type: ("Poetry", Path, Path, str) -> str
+        cls,
+        poetry: Poetry,
+        directory: Path | None = None,
+        original: Path | None = None,
+        executable: Path | None = None,
+        editable: bool = False,
+    ) -> str:
         wb = WheelBuilder(
-            poetry, target_dir=directory, original=original, executable=executable
+            poetry,
+            original=original,
+            executable=executable,
+            editable=editable,
         )
-        wb.build()
+        wb.build(target_dir=directory)
 
         return wb.wheel_filename
 
     @classmethod
-    def make(cls, poetry, executable=None):  # type: ("Poetry", Optional[str]) -> None
+    def make(cls, poetry: Poetry, executable: Path | None = None) -> None:
         """Build a wheel in the dist/ directory, and optionally upload it."""
         cls.make_in(poetry, executable=executable)
 
-    def build(self):  # type: () -> None
+    def build(
+        self,
+        target_dir: Path | None = None,
+    ) -> Path:
         logger.info("Building wheel")
 
-        dist_dir = self._target_dir
-        if not dist_dir.exists():
-            dist_dir.mkdir()
+        target_dir = target_dir or self.default_target_dir
+        if not target_dir.exists():
+            target_dir.mkdir()
 
         (fd, temp_path) = tempfile.mkstemp(suffix=".whl")
 
@@ -91,28 +101,50 @@ class WheelBuilder(Builder):
         new_mode = normalize_file_permissions(st_mode)
         os.chmod(temp_path, new_mode)
 
-        with os.fdopen(fd, "w+b") as fd_file:
-            with zipfile.ZipFile(
-                fd_file, mode="w", compression=zipfile.ZIP_DEFLATED
-            ) as zip_file:
+        with os.fdopen(fd, "w+b") as fd_file, zipfile.ZipFile(
+            fd_file, mode="w", compression=zipfile.ZIP_DEFLATED
+        ) as zip_file:
+            if not self._editable:
                 if not self._poetry.package.build_should_generate_setup():
                     self._build(zip_file)
                     self._copy_module(zip_file)
                 else:
                     self._copy_module(zip_file)
                     self._build(zip_file)
+            else:
+                self._build(zip_file)
+                self._add_pth(zip_file)
 
-                self._write_metadata(zip_file)
-                self._write_record(zip_file)
+            self._copy_file_scripts(zip_file)
+            self._write_metadata(zip_file)
+            self._write_record(zip_file)
 
-        wheel_path = dist_dir / self.wheel_filename
+        wheel_path = target_dir / self.wheel_filename
         if wheel_path.exists():
             wheel_path.unlink()
         shutil.move(temp_path, str(wheel_path))
 
-        logger.info("Built {}".format(self.wheel_filename))
+        logger.info(f"Built {self.wheel_filename}")
+        return wheel_path
 
-    def _build(self, wheel):  # type: (zipfile.ZipFile) -> None
+    def _add_pth(self, wheel: zipfile.ZipFile) -> None:
+        paths = set()
+        for include in self._module.includes:
+            if isinstance(include, PackageInclude) and (
+                include.is_module() or include.is_package()
+            ):
+                paths.add(include.base.resolve().as_posix())
+
+        content = ""
+        for path in paths:
+            content += path + os.linesep
+
+        pth_file = Path(self._module.name).with_suffix(".pth")
+
+        with self._write_to_zip(wheel, str(pth_file)) as f:
+            f.write(content)
+
+    def _build(self, wheel: zipfile.ZipFile) -> None:
         if self._package.build_script:
             if not self._poetry.package.build_should_generate_setup():
                 # Since we have a build script but no setup.py generation is required,
@@ -137,14 +169,14 @@ class WheelBuilder(Builder):
                         os.chdir(current_path)
 
                     build_dir = self._path / "build"
-                    lib = list(build_dir.glob("lib.*"))
-                    if not lib:
+                    libs: list[Path] = list(build_dir.glob("lib.*"))
+                    if not libs:
                         # The result of building the extensions
                         # does not exist, this may due to conditional
                         # builds, so we assume that it's okay
                         return
 
-                    lib = lib[0]
+                    lib = libs[0]
 
                     for pkg in lib.glob("**/*"):
                         if pkg.is_dir() or self.is_excluded(pkg):
@@ -155,11 +187,21 @@ class WheelBuilder(Builder):
                         if rel_path in wheel.namelist():
                             continue
 
-                        logger.debug("Adding: {}".format(rel_path))
+                        logger.debug(f"Adding: {rel_path}")
 
                         self._add_file(wheel, pkg, rel_path)
 
-    def _run_build_command(self, setup):  # type: (Path) -> None
+    def _copy_file_scripts(self, wheel: zipfile.ZipFile) -> None:
+        file_scripts = self.convert_script_files()
+
+        for abs_path in file_scripts:
+            self._add_file(
+                wheel,
+                abs_path,
+                Path.joinpath(Path(self.wheel_data_folder), "scripts", abs_path.name),
+            )
+
+    def _run_build_command(self, setup: Path) -> None:
         subprocess.check_call(
             [
                 self.executable.as_posix(),
@@ -170,19 +212,19 @@ class WheelBuilder(Builder):
             ]
         )
 
-    def _run_build_script(self, build_script):  # type: (str) -> None
-        logger.debug("Executing build script: {}".format(build_script))
+    def _run_build_script(self, build_script: str) -> None:
+        logger.debug(f"Executing build script: {build_script}")
         subprocess.check_call([self.executable.as_posix(), build_script])
 
-    def _copy_module(self, wheel):  # type: (zipfile.ZipFile) -> None
+    def _copy_module(self, wheel: zipfile.ZipFile) -> None:
         to_add = self.find_files_to_add()
 
         # Walk the files and compress them,
         # sorting everything so the order is stable.
-        for file in sorted(list(to_add), key=lambda x: x.path):
+        for file in sorted(to_add, key=lambda x: x.path):
             self._add_file(wheel, file.path, file.relative_to_source_root())
 
-    def _write_metadata(self, wheel):  # type: (zipfile.ZipFile) -> None
+    def _write_metadata(self, wheel: zipfile.ZipFile) -> None:
         if (
             "scripts" in self._poetry.local_config
             or "plugins" in self._poetry.local_config
@@ -199,10 +241,10 @@ class WheelBuilder(Builder):
 
         for path in set(license_files_to_add):
             if path.is_file():
-                relative_path = "%s/%s" % (self.dist_info, path.relative_to(self._path))
+                relative_path = f"{self.dist_info}/{path.relative_to(self._path)}"
                 self._add_file(wheel, path, relative_path)
             else:
-                logger.debug("Skipping: {}".format(path.as_posix()))
+                logger.debug(f"Skipping: {path.as_posix()}")
 
         with self._write_to_zip(wheel, self.dist_info + "/WHEEL") as f:
             self._write_wheel_file(f)
@@ -210,10 +252,10 @@ class WheelBuilder(Builder):
         with self._write_to_zip(wheel, self.dist_info + "/METADATA") as f:
             self._write_metadata_file(f)
 
-    def _write_record(self, wheel):  # type: (zipfile.ZipFile) -> None
+    def _write_record(self, wheel: zipfile.ZipFile) -> None:
         # Write a record of the files in the wheel
         with self._write_to_zip(wheel, self.dist_info + "/RECORD") as f:
-            record = StringIO() if not PY2 else BytesIO()
+            record = StringIO()
 
             csv_writer = csv.writer(
                 record,
@@ -222,41 +264,43 @@ class WheelBuilder(Builder):
                 lineterminator="\n",
             )
             for path, hash, size in self._records:
-                csv_writer.writerow((path, "sha256={}".format(hash), size))
+                csv_writer.writerow((path, f"sha256={hash}", size))
 
             # RECORD itself is recorded with no hash or size
             csv_writer.writerow((self.dist_info + "/RECORD", "", ""))
 
-            f.write(decode(record.getvalue()))
+            f.write(record.getvalue())
 
     @property
-    def dist_info(self):  # type: () -> str
+    def dist_info(self) -> str:
         return self.dist_info_name(self._package.name, self._meta.version)
 
     @property
-    def wheel_filename(self):  # type: () -> str
-        return "{}-{}-{}.whl".format(
-            escape_name(self._package.pretty_name),
-            escape_version(self._meta.version),
-            self.tag,
-        )
+    def wheel_data_folder(self) -> str:
+        return f"{self._package.name}-{self._meta.version}.data"
 
-    def supports_python2(self):  # type: () -> bool
+    @property
+    def wheel_filename(self) -> str:
+        name = escape_name(self._package.pretty_name)
+        version = escape_version(self._meta.version)
+        return f"{name}-{version}-{self.tag}.whl"
+
+    def supports_python2(self) -> bool:
         return self._package.python_constraint.allows_any(
             parse_constraint(">=2.0.0 <3.0.0")
         )
 
-    def dist_info_name(self, distribution, version):  # type: (str, str) -> str
+    def dist_info_name(self, distribution: str, version: str) -> str:
         escaped_name = escape_name(distribution)
         escaped_version = escape_version(version)
 
-        return "{}-{}.dist-info".format(escaped_name, escaped_version)
+        return f"{escaped_name}-{escaped_version}.dist-info"
 
     @property
-    def tag(self):  # type: () -> str
+    def tag(self) -> str:
         if self._package.build_script:
-            tag = next(sys_tags())
-            tag = (tag.interpreter, tag.abi, tag.platform)
+            sys_tag = next(sys_tags())
+            tag = (sys_tag.interpreter, sys_tag.abi, sys_tag.platform)
         else:
             platform = "any"
             if self.supports_python2():
@@ -269,8 +313,11 @@ class WheelBuilder(Builder):
         return "-".join(tag)
 
     def _add_file(
-        self, wheel, full_path, rel_path
-    ):  # type: (zipfile.ZipFile, Union[Path, str], Union[Path, str]) -> None
+        self,
+        wheel: zipfile.ZipFile,
+        full_path: Path | str,
+        rel_path: Path | str,
+    ) -> None:
         full_path, rel_path = str(full_path), str(rel_path)
         if os.sep != "/":
             # We always want to have /-separated paths in the zip file and in
@@ -305,8 +352,8 @@ class WheelBuilder(Builder):
 
     @contextlib.contextmanager
     def _write_to_zip(
-        self, wheel, rel_path
-    ):  # type: (zipfile.ZipFile, str) -> Iterator[StringIO]
+        self, wheel: zipfile.ZipFile, rel_path: str
+    ) -> Iterator[StringIO]:
         sio = StringIO()
         yield sio
 
@@ -323,20 +370,20 @@ class WheelBuilder(Builder):
         wheel.writestr(zi, b, compress_type=zipfile.ZIP_DEFLATED)
         self._records.append((rel_path, hash_digest, len(b)))
 
-    def _write_entry_points(self, fp):  # type: (TextIO) -> None
+    def _write_entry_points(self, fp: TextIO) -> None:
         """
         Write entry_points.txt.
         """
         entry_points = self.convert_entry_points()
 
         for group_name in sorted(entry_points):
-            fp.write("[{}]\n".format(group_name))
+            fp.write(f"[{group_name}]\n")
             for ep in sorted(entry_points[group_name]):
                 fp.write(ep.replace(" ", "") + "\n")
 
             fp.write("\n")
 
-    def _write_wheel_file(self, fp):  # type: (TextIO) -> None
+    def _write_wheel_file(self, fp: TextIO) -> None:
         fp.write(
             wheel_file_template.format(
                 version=__version__,
@@ -345,8 +392,8 @@ class WheelBuilder(Builder):
             )
         )
 
-    def _write_metadata_file(self, fp):  # type: (TextIO) -> None
+    def _write_metadata_file(self, fp: TextIO) -> None:
         """
         Write out metadata in the 2.x format (email like)
         """
-        fp.write(decode(self.get_metadata_content()))
+        fp.write(self.get_metadata_content())
